@@ -5,7 +5,7 @@ use strict;
 use DBI   1.57 ();
 use DynaLoader ();
 
-our $VERSION = '1.40';
+our $VERSION = '1.42';
 our @ISA     = 'DynaLoader';
 
 # sqlite_version cache (set in the XS bootstrap)
@@ -95,15 +95,24 @@ sub connect {
             my ($key, $value) = split(/=/, $attrib, 2);
             if ( $key =~ /^(?:db(?:name)?|database)$/ ) {
                 $real = $value;
+            } elsif ( $key eq 'uri' ) {
+                $real = $value;
+                $attr->{sqlite_open_flags} |= DBD::SQLite::OPEN_URI();
             } else {
                 $attr->{$key} = $value;
             }
         }
     }
 
+    if (my $flags = $attr->{sqlite_open_flags}) {
+        unless ($flags & (DBD::SQLite::OPEN_READONLY() | DBD::SQLite::OPEN_READWRITE())) {
+            $attr->{sqlite_open_flags} |= DBD::SQLite::OPEN_READWRITE() | DBD::SQLite::OPEN_CREATE();
+        }
+    }
+
     # To avoid unicode and long file name problems on Windows,
     # convert to the shortname if the file (or parent directory) exists.
-    if ( $^O =~ /MSWin32/ and $real ne ':memory:' and $real ne '') {
+    if ( $^O =~ /MSWin32/ and $real ne ':memory:' and $real ne '' and $real !~ /^file:/) {
         require Win32;
         require File::Basename;
         my ($file, $dir, $suffix) = File::Basename::fileparse($real);
@@ -985,6 +994,23 @@ If the filename C<$dbfile> is an empty string, then a private,
 temporary on-disk database will be created. This private database will
 be automatically deleted as soon as the database connection is closed.
 
+As of 1.41_01, you can pass URI filename (see L<http://www.sqlite.org/uri.html>)
+as well for finer control:
+
+  my $dbh = DBI->connect("dbi:SQLite:uri=file:$path_to_dbfile?mode=rwc");
+
+Note that this is not for remote SQLite database connection. You only can
+connect to a local database.
+
+You can also set sqlite_open_flags (only) when you connect to a database:
+
+  use DBD::SQLite;
+  my $dbh = DBI->connect("dbi:SQLite:$dbfile", undef, undef, {
+    sqlite_open_flags => DBD::SQLite::OPEN_READONLY,
+  });
+
+See L<http://www.sqlite.org/c3ref/open.html> for details.
+
 =head2 DBD::SQLite And File::Temp
 
 When you use L<File::Temp> to create a temporary file/directory for
@@ -1123,6 +1149,20 @@ problems. Use this sparingly when you handle existing databases.
 If you handle databases created by other tools like native C<sqlite3>
 command line tool, this attribute would help you.
 
+As of 1.41_04, C<sqlite_see_if_its_a_number> works only for
+bind values with no explicit type.
+
+  my $dbh = DBI->connect('dbi:SQLite:foo', undef, undef, {
+    AutoCommit => 1,
+    RaiseError => 1,
+    sqlite_see_if_its_a_number => 1,
+  });
+  my $sth = $dbh->prepare('INSERT INTO foo VALUES(?)');
+  # '1.230' will be inserted as a text, instead of 1.23 as a number,
+  # even though sqlite_see_if_its_a_number is set.
+  $sth->bind_param(1, '1.230', SQL_VARCHAR);
+  $sth->execute;
+
 =back
 
 =head2 Placeholders
@@ -1574,6 +1614,47 @@ Whether the column is primary or unique.
 B<Note>: foreign key support in SQLite must be explicitly turned on through
 a C<PRAGMA> command; see L</"Foreign keys"> earlier in this manual.
 
+=head2 statistics_info
+
+  $sth = $dbh->statistics_info(undef, $schema, $table,
+                                $unique_only, $quick);
+
+Returns information about a table and it's indexes, as specified in
+L<DBI/statistics_info>, but with some limitations : 
+
+=over
+
+=item *
+
+information in rows returned by the C<$sth> is incomplete with
+respect to the L<DBI/statistics_info> specification. All requested fields
+are present, but the content is C<undef> for some of them.
+
+=back
+
+The following nonempty fields are returned :
+
+B<TABLE_SCHEM>:
+The name of the schema (database) that the table is in. The default schema is 'main', temporary tables are in 'temp' and other databases will be in the name given when the database was attached.
+
+B<TABLE_NAME>:
+The name of the table
+
+B<NON_UNIQUE>:
+Contains 0 for unique indexes, 1 for non-unique indexes
+
+B<INDEX_NAME>:
+The name of the index
+
+B<TYPE>:
+SQLite uses 'btree' for all it's indexes
+
+B<ORDINAL_POSITION>:
+Column sequence number (starting with 1).
+
+B<COLUMN_NAME>:
+The name of the column
+
 =head2 ping
 
   my $bool = $dbh->ping;
@@ -2271,11 +2352,11 @@ Here is a very short example of using FTS :
   $dbh->do(<<"") or die DBI::errstr;
   CREATE VIRTUAL TABLE fts_example USING fts4(content)
   
-  my $sth = $dbh->prepare("INSERT INTO fts_example(content) VALUES (?))");
+  my $sth = $dbh->prepare("INSERT INTO fts_example(content) VALUES (?)");
   $sth->execute($_) foreach @docs_to_insert;
   
   my $results = $dbh->selectall_arrayref(<<"");
-  SELECT docid, snippet(content) FROM fts_example WHERE content MATCH 'foo'
+  SELECT docid, snippet(fts_example) FROM fts_example WHERE content MATCH 'foo'
   
 
 The key points in this example are :
@@ -2568,10 +2649,6 @@ code we work with leaks.
 
 Reading/writing into blobs using C<sqlite2_blob_open> / C<sqlite2_blob_close>.
 
-=head2 Flags for sqlite3_open_v2
-
-Support the full API of sqlite3_open_v2 (flags for opening the file).
-
 =head2 Support for custom callbacks for R-Tree queries
 
 Custom queries of a R-Tree index using a callback are possible with
